/********************************************************************\

  Name:         histo.c
  Created by:   Stefan Ritt
  Contents:     General purpos histogram booking/filling module

  $Log$
\********************************************************************/

/*-- Include files -------------------------------------------------*/

/* standard includes */
#include <stdio.h>
#include <math.h>

/* midas includes */
#include "midas.h"
#include "experim.h"
#include "analyzer.h"
#include "test.h"

/* cernlib includes */
#ifdef OS_WINNT
#define VISUAL_CPLUSPLUS
#endif

#ifdef OS_LINUX
#define f2cFortran
#endif

#include <cfortran.h>
#include <hbook.h>

#ifndef PI
#define PI 3.14159265359
#endif

/*-- Parameters ----------------------------------------------------*/

HISTO_PARAM       histo_param;
extern EXP_PARAM  exp_param;

/*-- Globals -------------------------------------------------------*/

static int n_events, n_hits_wc1, n_hits_wc2;

/*-- Module declaration --------------------------------------------*/

INT histo_filling(EVENT_HEADER*,void*);
INT histo_booking(void);

HISTO_PARAM_STR(histo_param_str);

ANA_MODULE histo_module = {
  "General histo",               /* module name           */
  "Stefan Ritt",                 /* author                */
  histo_filling,                 /* event routine         */
  NULL,                          /* BOR routine           */
  NULL,                          /* EOR routine           */
  histo_booking,                 /* init routine          */
  NULL,                          /* exit routine          */
  &histo_param,                  /* parameter structure   */
  sizeof(histo_param),           /* structure size        */
  histo_param_str,               /* initial parameters    */
};

/*-- init routine --------------------------------------------------*/


INT histo_booking(void)
{
int i;
char str[80];

  /* PCOS chambers */

  HBOOK1(HISTO_ID_BASE+0, "AX", 64, -64.f, 64.f, 0.f);
  HBOOK1(HISTO_ID_BASE+1, "AY", 64, -64.f, 64.f, 0.f);
  HBOOK1(HISTO_ID_BASE+2, "BX", 64, -64.f, 64.f, 0.f);
  HBOOK1(HISTO_ID_BASE+3, "BY", 64, -64.f, 64.f, 0.f);
  
  HBOOK2(HISTO_ID_BASE+4, "A", 64, -64.f, 64.f, 64, -64.f, 64.f, 0.f);
  HBOOK2(HISTO_ID_BASE+5, "B", 64, -64.f, 64.f, 64, -64.f, 64.f, 0.f);

  n_events = n_hits_wc1 = n_hits_wc2 = 0;

  /* wire ADC ratios */

  for (i=0 ; i<4 ; i++)
    {
    sprintf(str, "WP1sum%d", i+1);
    HBOOK1(HISTO_ID_BASE+100+i, str, 1000, 0.f, 2000.f, 0.f);
    sprintf(str, "WP2sum%d", i+1);
    HBOOK1(HISTO_ID_BASE+100+i+4, str, 1000, 0.f, 2000.f, 0.f);

    sprintf(str, "WP1ratio%d", i+1);
    HBOOK2(HISTO_ID_BASE+200+i, str, 50, 0.f, 2000.f, 50, 0.f, 1.f, 0.f);
    sprintf(str, "WP2ratio%d", i+1);
    HBOOK2(HISTO_ID_BASE+200+i+4, str, 50, 0.f, 2000.f, 50, 0.f, 1.f, 0.f);
    }

  return SUCCESS;
}

/*-- event routine -------------------------------------------------*/

INT histo_filling(EVENT_HEADER *pheader, void *pevent)
{
float        *cadc;
float        *ctdc;
PCOS_BANK    *pcos;
WIRE_BANK    *wire;
STRP_BANK    *strp;
float        effi_1, effi_2;
HNDLE        hDB;
int          i, n_pcos, n_adc, n_tdc;
float        wp1_sum[4], wp2_sum[4], wp1_ratio[4], wp2_ratio[4];

  /* open banks of interest */
  n_pcos = bk_locate(pevent, "PCOS", &pcos);
  n_adc = bk_locate(pevent, "CADC", &cadc);
  n_tdc = bk_locate(pevent, "CTDC", &ctdc);

  cm_get_experiment_database(&hDB, NULL);

  /*---- fill wire chamber histos ----------------------------------*/

  if (n_pcos)
    {
    HF1(HISTO_ID_BASE+0, pcos->ax, 1.f);
    HF1(HISTO_ID_BASE+1, pcos->ay, 1.f);
    HF1(HISTO_ID_BASE+2, pcos->bx, 1.f);
    HF1(HISTO_ID_BASE+3, pcos->by, 1.f);

    HF2(HISTO_ID_BASE+4, pcos->ax, pcos->ay, 1.f);
    HF2(HISTO_ID_BASE+5, pcos->bx, pcos->by, 1.f);
    }

  /*---- calculate wire chamber efficency --------------------------*/

  if (n_pcos)
    {
    n_events++;

    if (pcos->ax != INVALID &&
        pcos->ay != INVALID)
      n_hits_wc1++;
    if (pcos->bx != INVALID &&
        pcos->by != INVALID)
      n_hits_wc2++;

    if (n_events >= 2500)  // 2% error
      {
      effi_1 = (float) n_hits_wc1 / n_events;
      effi_2 = (float) n_hits_wc2 / n_events;
    
      n_events = n_hits_wc1 = n_hits_wc2 = 0;

      db_set_value(hDB, 0, "/Equipment/Trigger/Variables/Effi 1", &effi_1, sizeof(float), 1, TID_FLOAT);
      db_set_value(hDB, 0, "/Equipment/Trigger/Variables/Effi 2", &effi_2, sizeof(float), 1, TID_FLOAT);
      }
    }
  
  /*---- calculate wire ADC ratios ---------------------------------*/

  memset(wp1_ratio, 0, sizeof(wp1_ratio));
  memset(wp2_ratio, 0, sizeof(wp2_ratio));

  /* create wire sum bank */
  bk_create(pevent, "WIRE", TID_STRUCT, &wire);

  for (i=0 ; i<4 ; i++)
    {
    wp1_sum[i] = cadc[i+0]+cadc[i+4];
    if (wp1_sum[i] > 0)
      wp1_ratio[i] = cadc[i+0] / wp1_sum[i];

    wp2_sum[i] = cadc[i+8]+cadc[i+13];
    if (wp2_sum[i] > 0)
      wp2_ratio[i] = cadc[i+8] / wp1_sum[i];
    }

  wire->w1_sum = wp1_sum[0];
  wire->w2_sum = wp1_sum[1];
  wire->w3_sum = wp1_sum[2];
  wire->w4_sum = wp1_sum[3];

  bk_close(pevent, wire+1);

  /* fill histos */
  for (i=0 ; i<4 ; i++)
    {
    if (wp1_sum[i] > histo_param.histo_threshold)
      HF1(HISTO_ID_BASE+100+i, wp1_sum[i], 1.f);
    if (wp2_sum[i] > histo_param.histo_threshold)
      HF1(HISTO_ID_BASE+100+i+4, wp2_sum[i], 1.f);

    if (wp1_sum[i] > histo_param.histo_threshold)
      HF2(HISTO_ID_BASE+200+i, wp1_sum[i], wp1_ratio[i], 1.f);
    if (wp2_sum[i] > histo_param.histo_threshold)
      HF2(HISTO_ID_BASE+200+i+4, wp2_sum[i], wp2_ratio[i], 1.f);
    }

  /* set efficiency tests */
  SET_TEST(w1, wp1_sum[0] > histo_param.histo_threshold);
  SET_TEST(w2, wp1_sum[1] > histo_param.histo_threshold);
  SET_TEST(w3, wp1_sum[2] > histo_param.histo_threshold);
  SET_TEST(w4, wp1_sum[3] > histo_param.histo_threshold);

  SET_TEST(wp1_or, TEST(w1) | TEST(w1) | TEST(w3) | TEST(w4));

  /*---- create strip ADC bank -------------------------------------*/

  bk_create(pevent, "STRP", TID_STRUCT, &strp);

  for (i=0 ; i<12 ; i++)
    *((&strp->s1)+i) = cadc[36+i];

  bk_close(pevent, strp+1);


  return ANA_CONTINUE;
}
